'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var isValidElement = React.isValidElement, cloneElement = React.cloneElement;
function replaceTree(input, props, key, state) {
    if (props.updateState) {
        state = props.updateState(input, state);
    }
    var result = input;
    if (isValidElement(input)) {
        var element = input;
        if (typeof element.type === "function") {
            if ('prototype' in element.type && element.type.prototype.isReactComponent) {
                result = new element.type(element.props).render() || React.createElement(React.Fragment, null);
            }
            else {
                result = element.type(element.props);
            }
        }
        else {
            result = element;
        }
        result = cloneElement(result, __assign({}, result.props, { key: key, children: replaceTree(result.props.children, props, key, state) }));
    }
    else if (Array.isArray(input)) {
        result = input.reduce(function (acc, item, index) { return acc.concat(replaceTree(item, props, key ? key + "-" + index : index, state)); }, []);
    }
    if (props.match(input, state)) {
        result = props.replace(input, state);
        result = cloneElement(result, __assign({}, result.props, { key: key }));
    }
    return result;
}
function Replacer(props) {
    var match = function () { return false; };
    var replace = props.replace;
    var updateState = props.updateState;
    if ("matchElement" in props) {
        match = function (i) { return isValidElement(i) && i.type === props.matchElement; };
    }
    else if ("matchLiteral" in props) {
        match = function (i) {
            return !Array.isArray(i) && !isValidElement(i) && props.matchLiteral(i);
        };
    }
    else {
        match = props.match;
    }
    return replaceTree(props.children, { match: match, replace: replace, updateState: updateState }, undefined, props.initialState);
}

exports.Replacer = Replacer;
//# sourceMappingURL=index.js.map
